; hacker port firmware
; version 0.1 date 2004-05-05
; part of the hacker port project - http[s]://1stein.schuermans.info/hackerport/
; Copyright (C) 2003-2004 1stein <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html

	TITLE		"hackerport firmware"

	include		"P18F442.INC"

	include		"IoPins.inc"
	include		"Bits.inc"
	include		"Version.inc"



;===========================================================================================
; imported variables
;===========================================================================================



;===========================================================================================
; imported procedures
;===========================================================================================
	extern		USB_INIT, USB_TASK
	extern		DEBUG_INIT



;===========================================================================================
accessram	UDATA_ACS		; access RAM
;===========================================================================================

; storage for global bits (see Bits.inc)
BITS0		res	1

; current state of latched port (see IoPins.inc)
LATCHED_PORT	res	1

; general purpose registers
A1_REG		res	1
B1_REG		res	1
C1_REG		res	1
D1_REG		res	1
A2_REG		res	2
B2_REG		res	2
C2_REG		res	2
D2_REG		res	2
A4_REG		res	4


; buffer for output port values and PWM values
OUT_PWM_BUF	res	0
OUT_A_VAL	res	1		; values for output ports
OUT_B_VAL	res	1
PWM_A_VAL	res	2		; values for PWM outputs (big endian)
PWM_B_VAL	res	2

; buffer for input port values and analog input values
IN_ANA_BUF	res	0
IN_A_VAL	res	1		; values from input ports
IN_B_VAL	res	1
ANA_A_VAL	res	2		; values from analog input (big endian)
ANA_B_VAL	res	2
ANA_C_VAL	res	2
ANA_D_VAL	res	2
ANA_E_VAL	res	2



;===========================================================================================
ram_main	UDATA			; normal RAM for main
;===========================================================================================

; backup registers for low priority interrupts
INT_WREG	res	1
INT_STATUS	res	1
INT_BSR		res	1



;===========================================================================================
entry		CODE			; code entry point
;===========================================================================================
ENTRY:
	nop				; used by ICD
	nop
	goto	INIT

;===========================================================================================
int_hi		CODE			; high priority interrupts
;===========================================================================================
HIGH_INTERRUPT:
	reset				; something went completely wrong, so start over

;===========================================================================================
int_lo		CODE			; low priority interrupts
;===========================================================================================
LOW_INTERRUPT:
	reset				; something went completely wrong, so start over



;===========================================================================================
prog		CODE			; normal code
;===========================================================================================

;-------------------------------------------------------------------------------------------
; initialization
;-------------------------------------------------------------------------------------------
INIT:
; set default values for output pins
	bcf	PIN_LATCH_LE		; clear latched port latch enable
	bcf	PIN_OUT_A_LE		; clear OUTPUT A latch enable
	bcf	PIN_OUT_B_LE		; clear OUTPUT B latch enable
	bsf	PIN_IN_A_nOE		; clear INPUT A output enable
	bsf	PIN_IN_B_nOE		; clear INPUT B output enable
	clrf	PORT_BUS_O		; set data bus to 0x00
	bcf	PIN_PIC_PGD_O		; clear PICPROG PGD
	bsf	PIN_USB_nRST		; clear USBN9604 reset
	bcf	PIN_USB_A0		; clear USBN9604 address bit
	bsf	PIN_USB_nRD		; set USBN9604 read
	bsf	PIN_USB_nWR		; set USBN9604 write
	bsf	PIN_USB_nCS		; set USBN9604 chis select
; set up analog inputs
	movlw	b'10000001'		; FOSC/64 = 1.6us at 40MHz, select AN0
	movwf	ADCON0			; no conversion active, analog input module powered up
	movlw	b'01000010'		; left justified values, AN0..AN4 (RA0..3+RA5) active
	movwf	ADCON1
; define inputs/outputs
	setf	TRISA			; RA0..6 input (RA7 does not exist)
	clrf	TRISB			; RB0..5 output (RB6..7 for ICD2 connection)
	movlw	b'10000000'		; RC0+RC5 output, RC1..2 (PWM) output
	movwf	TRISC			; RC3+RC4 (SCL+SDA) output, RC6 (TX) output, RC7 (RX) input
	setf	TRISD			; RD0..7 input
	clrf	TRISE			; RE0..2 output (RE3..7 do not exist), parallel slave port off
; initialize timers
	movlw	b'01111111'		; timer 0 off
	movwf	T0CON
	movlw	b'00000000'		; timer 1 off
	movwf	T1CON
	movlw	b'00000110'		; timer 2 on, prescaler 1:16, postscaler 1:1
	movwf	T2CON
	movlw	249			; timer 2: 16*250=4000 cycles, period = 2.5 kHz at 40 MHz
	movwf	PR2
	movlw	b'00000000'		; timer 3 off
	movwf	T3CON
; set up PWM outputs
	clrf	CCPR1L			; CCP1: duty cycle 0
	movlw	b'00001100'		; CCP1: PWM mode
	movwf	CCP1CON
	clrf	CCPR2L			; CCP2: duty cycle 0
	movlw	b'00001100'		; CCP2: PWM mode
	movwf	CCP2CON
; set up I2C in master mode
	movlw	b'10000000'		; no slew rate control, SMBus off, clear status flags
	movwf	SSPSTAT
	movlw	b'00101000'		; no collision yet, enable MSSP, master mode
	movwf	SSPCON1
	movlw	b'00000000'		; no acknowledge yet, recv off, stop off, repeated start off, start off
	movwf	SSPCON2
	movlw	0x3F			; 100kHz
	movwf	SSPADD
; set up USART
	movlw	b'00000110'		; 8 bit, async mode, high speed, shift reg empty
	movwf	TXSTA
	movlw	b'10010000'		; port enabled, 8 bit, recv enabled, no addr detect
	movwf	RCSTA
	movlw	22			; 115200 kbps (1,4% error)
	movwf	SPBRG
; initialize latched port
	bcf	PINL_JTAG_TRST		; JTAG bits to 0
	bcf	PINL_JTAG_TCLK
	bcf	PINL_JTAG_TMS
	bcf	PINL_JTAG_TDI
	bcf	PINL_PIC_VPP		; PICPROG bits to 0
	bcf	PINL_PIC_MCLR
	bcf	PINL_PIC_PGC
	bcf	PINL_LED_13V		; USB LED / 13V generator clock on
	call	LATCHED_PORT_OUT
; initialize output ports
	clrf	PORT_BUS_O		; output 0x00 on data bus
	clrf	PORT_BUS_T		; set data bus port to output
	bsf	PIN_OUT_A_LE		; set output port A latch enable
	bcf	PIN_OUT_A_LE		; clear output port A latch enable
	bsf	PIN_OUT_B_LE		; set output port B latch enable
	bcf	PIN_OUT_B_LE		; clear output port B latch enable
	setf	PORT_BUS_T		; set data bus port back to input
; initialize PWM outputs
	clrf	CCPR1L			; PWM A to 0% duty cycle
	bcf	CCP1CON,DC1B1
	bcf	CCP1CON,DC1B0
	clrf	CCPR2L			; PWM B to 0% duty cycle
	bcf	CCP2CON,DC2B1
	bcf	CCP2CON,DC2B0
; initialize global variables
	clrf	BITS0
; call initialization procedures
	call	USB_INIT
	call	DEBUG_INIT
; initialize local variables
	setf	ANA_A_VAL+0		; impossible value (last 6 bits are always 0)
	setf	ANA_A_VAL+1		; - this leads to a change detection after first read
; initialize interrupts
;!!!
; reset watchdog
	clrwdt
; initialization done, start program
	goto	MAIN

;-------------------------------------------------------------------------------------------
; output value of LATCHED_PORT to latched port
; input:	LATCHED_PORT		new state for the latched port
; output:	-
; changes:	-
;-------------------------------------------------------------------------------------------
LATCHED_PORT_OUT:
	movff	LATCHED_PORT,PORT_BUS_O	; output data
	clrf	PORT_BUS_T		; set data bus port to output
	bsf	PIN_LATCH_LE		; set latched port latch enable
	bcf	PIN_LATCH_LE		; clear latched port latch enable
	setf	PORT_BUS_T		; set data bus port back to input
	return

;-------------------------------------------------------------------------------------------
; main program
;-------------------------------------------------------------------------------------------
MAIN:

; wait 1 ms
	movlw	10
	movwf	B1_REG
MAIN_WAIT:
	clrwdt
	movlw	200
	movwf	A1_REG
MAIN_WAIT_I:
	nop
	nop
	decfsz	A1_REG
	bra	MAIN_WAIT_I
	decfsz	B1_REG
	bra	MAIN_WAIT

; main loop
MAIN_LOOP:

; read input ports
	bcf	PIN_IN_A_nOE		; set input port A output enable
	movf	PORT_BUS_I,W		; read value
	bsf	PIN_IN_A_nOE		; clear input port A output enable
	cpfseq	IN_A_VAL		; if not equal to last value, ...
	bsf	BIT_IN_ANA		; ..., set change flag
	movwf	IN_A_VAL		; save new value (also if no change)
	bcf	PIN_IN_B_nOE		; set input port B output enable
	movf	PORT_BUS_I,W		; read value
	bsf	PIN_IN_B_nOE		; clear input port B output enable
	cpfseq	IN_B_VAL		; if not equal to last value, ...
	bsf	BIT_IN_ANA		; ..., set change flag
	movwf	IN_B_VAL		; save new value (also if no change)
; read analog inputs
	movlw	5			; load counter
	movwf	B1_REG
	clrf	C1_REG			; set analog input counter to AN0
	lfsr	FSR0,ANA_A_VAL		; get destination address
MAIN_ANA_LOOP:
	movlw	b'11000111'		; select analog input
	andwf	ADCON0
	movf	C1_REG,W
	iorwf	ADCON0
	bsf	ADCON0,GO		; start conversion (12 steps, 1.6us each => 19.2us)
	movlw	50			; wait for conversion to finish (25us at 40MHz)
	movwf	D1_REG
MAIN_ANA_WAIT:
	btfss	ADCON0,GO
	bra	MAIN_ANA_DONE		; end loop if conversion is done
	decfsz	D1_REG			; wait loop
	bra	MAIN_ANA_WAIT
	bsf	ADRESL,0		; set last bit to indicate timeout (last 6 bits are always 0)
MAIN_ANA_DONE:
	movf	ADRESH,W		; get high part of result
	cpfseq	INDF0			; if not equal to last value, ...
	bsf	BIT_IN_ANA		; ..., set change flag
	movwf	POSTINC0		; save new value (also if no change)
	movf	ADRESL,W		; get low part of result
	cpfseq	INDF0			; if not equal to last value, ...
	bsf	BIT_IN_ANA		; ..., set change flag
	movwf	POSTINC0		; save new value (also if no change)
	movlw	b'00001000'		; increment analog input counter
	addwf	C1_REG
	decfsz	B1_REG			; loop
	bra	MAIN_ANA_LOOP

; call USB
	call	USB_TASK

; output new values to output ports and PWM outputs
	btfss	BIT_OUT_PWM
	bra	MAIN_OUT_PWM_END	; jump if no new data available
	bcf	BIT_OUT_PWM		; reset change flag
; output new values to output ports
	movff	OUT_A_VAL,PORT_BUS_O	; output value for output port A on data bus
	clrf	PORT_BUS_T		; set data bus port to output
	bsf	PIN_OUT_A_LE		; set output port A latch enable
	bcf	PIN_OUT_A_LE		; clear output port A latch enable
	movff	OUT_B_VAL,PORT_BUS_O	; output value for output port B on data bus
	bsf	PIN_OUT_B_LE		; set output port B latch enable
	bcf	PIN_OUT_B_LE		; clear output port B latch enable
	setf	PORT_BUS_T		; set data bus port back to input
; write new values to PWM registers
	movff	PWM_A_VAL+0,CCPR1L	; PWM A
	bcf	CCP1CON,DC1B1
	btfsc	PWM_A_VAL+1,7
	bsf	CCP1CON,DC1B1
	bcf	CCP1CON,DC1B0
	btfsc	PWM_A_VAL+1,6
	bsf	CCP1CON,DC1B0
	movff	PWM_B_VAL+0,CCPR2L	; PWM B
	bcf	CCP2CON,DC2B1
	btfsc	PWM_B_VAL+1,7
	bsf	CCP2CON,DC2B1
	bcf	CCP2CON,DC2B0
	btfsc	PWM_B_VAL+1,6
	bsf	CCP2CON,DC2B0
MAIN_OUT_PWM_END:

; if we are allowed to draw power from USB, turn on LED and run 13V generator
	btfss	BIT_USB_PWR_OK
	bra	MAIN_LED_OFF		; jump if we may not draw power from USB
	btfsc	PINL_LED_13V
	bra	MAIN_LED_OFF		; jump if LED is on (13V generator needs toggling bit)
	bsf	PINL_LED_13V		; turn on LED
	bra	MAIN_LED_OUT
MAIN_LED_OFF:
	bcf	PINL_LED_13V		; turn off LED
MAIN_LED_OUT:
	call	LATCHED_PORT_OUT	; update state of latched port

; reset watchdog
	clrwdt
; go on with main loop
	bra	MAIN_LOOP



;===========================================================================================
; global variables
;===========================================================================================
	global		BITS0
	global		LATCHED_PORT
	global		A1_REG, B1_REG, C1_REG, D1_REG
	global		A2_REG, B2_REG, C2_REG, D2_REG
	global		A4_REG
	global		OUT_PWM_BUF, IN_ANA_BUF



;===========================================================================================
; global procedures
;===========================================================================================

	global		LATCHED_PORT_OUT



	end
